#!/sbin/itype
# This is a i file, used by initng parsed by install_service

# NAME: ALSA
# DESCRIPTION: The Advanced Linux Sound Architecture
# WWW: http://www.alsa-project.org/

service system/alsasound/cards {
	need = system/bootmisc;
	use = system/coldplug system/modules/depmod system/modules;
	script start = {
		# List of drivers for each card.
		for mod in `/sbin/modprobe -c | /bin/awk '$1 == "alias" && $2 ~ /^snd-card-[[:digit:]]$/ { print $2 } {}'`
		do
			/usr/sbin/ngc --instant --quiet -u system/modules/${mod}
		done

		# Fall back on the automated aliases,
		# if we do not have ALSA configured properly...
		[ -d /proc/asound ] && /bin/grep -q ' no soundcards ' /proc/asound/cards || exit 0

		echo "Could not detect custom ALSA settings.  Loading all detected alsa drivers."
		for mod in `/sbin/modprobe -c | /bin/awk '$2~ /^pci:/ && $3~ /^snd.*/ { print $3 }' | sort -u`
		do
			/usr/sbin/ngc --instant --quiet -u system/modules/${mod}
		done
	};
}

service system/alsasound/oss {
	need = system/bootmisc;
	use = system/coldplug system/modules/depmod system/modules;
	script start = {
		for mod in `/sbin/modprobe -l | /bin/grep "snd.*oss" | /bin/sed -e "s:\/.*\/::" -e "s:\..*::"`
		do
			/usr/sbin/ngc --instant --quiet -u system/modules/${mod}
		done
	};
}

service system/alsasound/seq {
	need = system/bootmisc;
	use = system/coldplug system/modules/depmod system/modules;
	script start = {
		# We want to ensure snd-seq is loaded as it is needed for things like
		# timidity even if we do not use a real sequencer.
		for mod in `/sbin/modprobe -l | /bin/sed -ne '{ s|.*/\([^/]*\)\.ko$|\1|; /^snd[_-]seq/ { /oss/ !p } }'`
		do
			/usr/sbin/ngc --instant --quiet -u system/modules/${mod}
		done
		for mod in `/bin/awk -F, '$2~ /^empty$/ { print $1 }' /proc/asound/seq/drivers`
		do
			/usr/sbin/ngc --instant --quiet -u system/modules/${mod}
		done
	};
}

service system/alsasound/ioctl32 {
	need = system/bootmisc;
	use = system/coldplug system/modules/depmod system/modules;
	script start = {
		# We want to ensure snd-ioctl32 is loaded as it is needed for 32bit
		# compatibility
		for mod in `/sbin/modprobe -l | /bin/sed -ne '{ s|.*/\([^/]*\)\.ko$|\1|; /^snd[_-]ioctl32/ p}'`
		do
			/usr/sbin/ngc --instant --quiet -u system/modules/${mod}
		done
	};
}

service system/alsasound/mixerstate {
	need = system/bootmisc system/alsasound;
	env asoundcfg=/etc/asound.state;
	script start = {
		if [ ! -r "${asoundcfg}" ]
		then
			echo "No mixer config in ${asoundcfg}, you have to unmute your card!"
			# this is not fatal!
		elif [ -x /usr/sbin/alsactl ]
		then
			for CARDNUM in `/bin/awk '/: / { print $1 }' /proc/asound/cards`
			do
				/usr/sbin/alsactl -f ${asoundcfg} restore ${CARDNUM} &
			done
			wait
		else
			echo "ERROR: Cannot find alsactl, did you forget to install media-sound/alsa-utils?"
			exit 1
		fi
		exit 0
	};
	exec stop = /usr/sbin/alsactl -f ${asoundcfg} store;
}

service system/alsasound {
	need = system/bootmisc;
	use = system/coldplug system/alsasound/cards system/alsasound/ioctl32 system/alsasound/seq system/alsasound/oss system/modules/depmod system/modules;
	also_stop = system/alsasound/cards system/alsasound/ioctl32 system/alsasound/seq system/alsasound/oss;
	env alsascrdir=/etc/alsa.d;
	script start = {
		for DRIVER in `/sbin/lsmod | /bin/awk '$1~/^snd.*/{print $1}'`
		do
			TMP=${DRIVER##snd-}
			TMP=${TMP##snd_}
			if [ -x "${alsascrdir}/${TMP}" ]
			then
				echo "  Running: ${alsascrdir}/${TMP} ..."
				${alsascrdir}/${TMP}
			fi
		done
	};
}
